local MASQUE = LibStub("Masque", true);
local buttongroupActionButtons = nil;
local buttongroupBags = nil;
    
local texture = "Interface\\PLAYERACTIONBARALT\\GENERICWOW";

function CleanUI_InitActionBars()

    CleanUserData.scaleActionBars = CleanUserData.scaleActionBars or 1.0

    if (MASQUE) then
        buttongroupActionButtons = MASQUE:Group("CleanUI", "Action Buttons");
        buttongroupBags = MASQUE:Group("CleanUI", "Bags");
        
        CleanUI_SkinActionBars();
        CleanUI_SkinBagButtons();
    end

    hooksecurefunc("ActionButton_OnUpdate", CleanUI_ActionButton_OnUpdate);
    hooksecurefunc("MainMenuTrackingBar_Configure", CleanUI_MainMenuTrackingBar_Configure);
    hooksecurefunc("PetActionBar_Update", CleanUI_InitPetBarArtwork);
    hooksecurefunc("PetActionBar_UpdatePositionValues", CleanUI_PetActionBar_UpdatePositionValues);
    hooksecurefunc("ActionButton_UpdateHotkeys", CleanUI_ActionButton_UpdateHotkeys);
    
    CleanUI_ReorderActionButtons();
    CleanUI_UpdateActionButtons();
    
    CleanUI_RemoveMacroNames();
    CleanUI_RemoveArtwork();
    CleanUI_InitActionBarArtwork();
    CleanUI_InitPetBarArtwork();
    CleanUI_ReorderExpBar();
    
    CleanUI_ScaleActionBars()
end

function CleanUI_UpdateActionButtons()
    for i = 1, 12 do
        CleanUI_ActionButton_UpdateHotkeys(_G["ActionButton"..i]);
        CleanUI_ActionButton_UpdateHotkeys(_G["MultiBarBottomLeftButton"..i]);
        CleanUI_ActionButton_UpdateHotkeys(_G["MultiBarBottomRightButton"..i]);
        CleanUI_ActionButton_UpdateHotkeys(_G["MultiBarRightButton"..i]);
        CleanUI_ActionButton_UpdateHotkeys(_G["MultiBarLeftButton"..i]);
    end
end

function CleanUI_ActionButton_UpdateHotkeys(self)
    local hotkey = self.HotKey;
    local text = hotkey:GetText()
    text = string.gsub( text, "Ziffernblock ", "Z" )
    hotkey:SetText(text)
end

function CleanUI_SetActionBarsScale(scale)
    if (scale and scale >= 0.5 and scale <= 1.5) then
        CleanUserData.scaleActionBars = scale
        CleanUI_ScaleActionBars() 
    else
        DEFAULT_CHAT_FRAME:AddMessage("scale: from 0.5 to 1.5", 1.0, 0, 0, 1, 10);
    end
end

function CleanUI_ScaleActionBars()    
    MainMenuBar:SetScale(CleanUserData.scaleActionBars);
end

function CleanUI_PetActionBar_UpdatePositionValues()
    if (PETACTIONBAR_XPOS >= 500) then
        PETACTIONBAR_XPOS = 323;
    end
end

function CleanUI_SkinActionBars()
    for i = 1, 12 do
        CleanUI_SkinActionButton(_G["ActionButton"..i]);
        CleanUI_SkinActionButton(_G["MultiBarBottomLeftButton"..i]);
        CleanUI_SkinActionButton(_G["MultiBarBottomRightButton"..i]);
        CleanUI_SkinActionButton(_G["MultiBarRightButton"..i]);
        CleanUI_SkinActionButton(_G["MultiBarLeftButton"..i]);
        
        _G["ActionButton"..i]:SetFrameStrata("MEDIUM");
        _G["MultiBarBottomLeftButton"..i]:SetFrameStrata("MEDIUM");
        _G["MultiBarBottomRightButton"..i]:SetFrameStrata("MEDIUM");
        _G["MultiBarRightButton"..i]:SetFrameStrata("MEDIUM");
        _G["MultiBarLeftButton"..i]:SetFrameStrata("MEDIUM");
    end

    for i = 1, 10 do
        CleanUI_SkinActionButton(_G["StanceButton"..i]);
        CleanUI_SkinActionButton(_G["PetActionButton"..i]);
    end
end

function CleanUI_SkinActionButton(button)
    buttongroupActionButtons:AddButton(button);
end

function CleanUI_SkinBagButtons()
    for i=0, 3, 1 do
        buttongroupBags:AddButton(_G["CharacterBag"..i.."Slot"]);
        local iconTexture = _G["CharacterBag"..i.."SlotIconTexture"];
        iconTexture:SetDrawLayer("BACKGROUND");
        iconTexture:SetTexCoord(0.08, 0.92, 0.08, 0.92);
        
        _G["CharacterBag"..i.."Slot"].IconBorder:SetTexture(nil);
        _G["CharacterBag"..i.."Slot"].IconBorder:SetSize(0.1, 0.1);
    end
    
    buttongroupBags:AddButton(MainMenuBarBackpackButton);
    local iconTexture = _G["MainMenuBarBackpackButtonIconTexture"];
    iconTexture:SetDrawLayer("BACKGROUND");
    iconTexture:SetTexCoord(0.08, 0.92, 0.08, 0.92);
end

function CleanUI_RemoveMacroNames()
    for i = 1, 12 do
        _G["ActionButton"..i.."Name"]:Hide();
        _G["MultiBarBottomLeftButton"..i.."Name"]:Hide();
        _G["MultiBarBottomRightButton"..i.."Name"]:Hide();
        _G["MultiBarRightButton"..i.."Name"]:Hide();
        _G["MultiBarLeftButton"..i.."Name"]:Hide();
    end
end

function CleanUI_RemoveArtwork()
    MainMenuBarLeftEndCap:SetTexture();
    MainMenuBarRightEndCap:SetTexture();
    MainMenuBarLeftEndCap:Hide();
    MainMenuBarRightEndCap:Hide();

    MainMenuMaxLevelBar0:SetTexture();
    MainMenuMaxLevelBar1:SetTexture();
    MainMenuMaxLevelBar2:SetTexture();
    MainMenuMaxLevelBar3:SetTexture();
    MainMenuMaxLevelBar0:Hide();
    MainMenuMaxLevelBar1:Hide();
    MainMenuMaxLevelBar2:Hide();
    MainMenuMaxLevelBar3:Hide();

    MainMenuBarTexture0:SetTexture();
    MainMenuBarTexture1:SetTexture();
    MainMenuBarTexture2:SetTexture();
    MainMenuBarTexture3:SetTexture();
    MainMenuBarTexture0:Hide();
    MainMenuBarTexture1:Hide();
    MainMenuBarTexture2:Hide();
    MainMenuBarTexture3:Hide();

    MainMenuXPBarTextureLeftCap:SetTexture();
    MainMenuXPBarTextureRightCap:SetTexture();
    MainMenuXPBarTextureMid:SetTexture();
    MainMenuXPBarTextureLeftCap:Hide();
    MainMenuXPBarTextureRightCap:Hide();
    MainMenuXPBarTextureMid:Hide();

    for i=1,19 do
        local texture = _G["MainMenuXPBarDiv"..i];
        texture:SetTexture();
        texture:Hide();
    end

    PossessBackground1:SetTexture();
    PossessBackground2:SetTexture();
    PossessBackground1:Hide();
    PossessBackground2:Hide();

    SlidingActionBarTexture0:SetTexture();
    SlidingActionBarTexture1:SetTexture();
    SlidingActionBarTexture0:Hide();
    SlidingActionBarTexture1:Hide();

    StanceBarLeft:SetTexture();
    StanceBarRight:SetTexture();
    StanceBarMiddle:SetTexture();
    StanceBarLeft:Hide();
    StanceBarRight:Hide();
    StanceBarMiddle:Hide();
    
    MainMenuBarPageNumber:Hide();
end

function CleanUI_InitActionBarArtwork()
    -- action bar
    local topleft = ActionButton1;

    if (MultiBarBottomLeft:IsShown()) then
        topleft = MultiBarBottomLeftButton1;
    end
    
    local cuiArtFrame = _G["CleanUIActionBarArtFrame"];
    if (not cuiArtFrame) then
        cuiArtFrame = CreateFrame("Frame", "CleanUIActionBarArtFrame", MainMenuBarArtFrame);
    end

    cuiArtFrame:SetParent(MainMenuBarArtFrame);
    cuiArtFrame:SetPoint("TOPLEFT", topleft, "TOPLEFT", -5, 5);
    cuiArtFrame:SetPoint("BOTTOMRIGHT", MainMenuBarArtFrame, "BOTTOMRIGHT", 0, 0);
    cuiArtFrame:SetFrameStrata("LOW");
    
    if (not cuiArtFrame.background) then
        cuiArtFrame.background = cuiArtFrame:CreateTexture("ARTWORK");
        cuiArtFrame.background:SetPoint("TOPLEFT", cuiArtFrame, "TOPLEFT", 0, 0);
        cuiArtFrame.background:SetPoint("BOTTOMRIGHT", cuiArtFrame, "BOTTOMRIGHT", 0, 0);
        cuiArtFrame.background:SetTexture(texture);
        cuiArtFrame.background:SetTexCoord(0.0, 1.0, 0.1757, 0.3555);
    end
    
    if (not cuiArtFrame.topborder) then
        cuiArtFrame.topborder = cuiArtFrame:CreateTexture("HIGHLIGHT");
        cuiArtFrame.topborder:SetPoint("TOPLEFT", cuiArtFrame, "TOPLEFT", 0, 0);
        cuiArtFrame.topborder:SetPoint("BOTTOMRIGHT", cuiArtFrame, "TOPRIGHT", 0, -3);
        cuiArtFrame.topborder:SetTexture(texture);
        cuiArtFrame.topborder:SetTexCoord(0.0, 1.0, 0.0898, 0.0977);
    end
    
    if (not cuiArtFrame.leftborder) then
        cuiArtFrame.leftborder = cuiArtFrame:CreateTexture("HIGHLIGHT");
        cuiArtFrame.leftborder:SetPoint("TOPLEFT", cuiArtFrame, "TOPLEFT", -115, 3);
        cuiArtFrame.leftborder:SetPoint("BOTTOMRIGHT", cuiArtFrame, "BOTTOMLEFT", 15, -3);
        cuiArtFrame.leftborder:SetTexture(texture);
        cuiArtFrame.leftborder:SetTexCoord(0, 0.3027, 0.7910, 1);
    end
    
    if (not cuiArtFrame.rightborder) then
        cuiArtFrame.rightborder = cuiArtFrame:CreateTexture("HIGHLIGHT");
        cuiArtFrame.rightborder:SetPoint("TOPLEFT", cuiArtFrame, "TOPRIGHT", -18, 3);
        cuiArtFrame.rightborder:SetPoint("BOTTOMRIGHT", cuiArtFrame, "BOTTOMRIGHT", 112, -3);
        cuiArtFrame.rightborder:SetTexture(texture);
        cuiArtFrame.rightborder:SetTexCoord(0.3027, 0, 0.7910, 1);
    end
end

function CleanUI_InitPetBarArtwork()    
    -- pet bar 
    local firstPetButton = _G["PetActionButton1"];
    local lastPetButton = _G["PetActionButton1"];
    local button;

    for i = 2, 10 do
        button = _G["PetActionButton"..i];
        if (button and button:IsVisible()) then
            lastPetButton = button;
        end
    end
    
    local cuiPetArtFrame = _G["CleanUIPetBarArtFrame"];
    if (not cuiPetArtFrame) then
        cuiPetArtFrame = CreateFrame("Frame", "CleanUIPetBarArtFrame", PetActionBarFrame);
    end

    cuiPetArtFrame:SetParent(PetActionBarFrame);
    cuiPetArtFrame:SetPoint("TOPLEFT", firstPetButton, "TOPLEFT", -12, 5);
    cuiPetArtFrame:SetPoint("BOTTOMRIGHT", lastPetButton, "BOTTOMRIGHT", 12, -3);
    cuiPetArtFrame:SetFrameStrata("LOW");
    
    if (not cuiPetArtFrame.background) then
        cuiPetArtFrame.background = cuiPetArtFrame:CreateTexture("ARTWORK");
        cuiPetArtFrame.background:SetPoint("TOPLEFT", cuiPetArtFrame, "TOPLEFT", 0, 0);
        cuiPetArtFrame.background:SetPoint("BOTTOMRIGHT", cuiPetArtFrame, "BOTTOMRIGHT", 0, 0);
        cuiPetArtFrame.background:SetTexture(texture);
        cuiPetArtFrame.background:SetTexCoord(0.0, 1.0, 0.1757, 0.3555);
    end    
    
    if (not cuiPetArtFrame.topborder) then
        cuiPetArtFrame.topborder = cuiPetArtFrame:CreateTexture("HIGHLIGHT");
        cuiPetArtFrame.topborder:SetPoint("TOPLEFT", cuiPetArtFrame, "TOPLEFT", 0, 0);
        cuiPetArtFrame.topborder:SetPoint("BOTTOMRIGHT", cuiPetArtFrame, "TOPRIGHT", 0, -3);
        cuiPetArtFrame.topborder:SetTexture(texture);
        cuiPetArtFrame.topborder:SetTexCoord(0.0, 1.0, 0.0898, 0.0977);
    end
    
    if (not cuiPetArtFrame.leftborder) then
        cuiPetArtFrame.leftborder = cuiPetArtFrame:CreateTexture("HIGHLIGHT");
        cuiPetArtFrame.leftborder:SetPoint("TOPLEFT", cuiPetArtFrame, "TOPLEFT", -2, 3);
        cuiPetArtFrame.leftborder:SetPoint("BOTTOMRIGHT", cuiPetArtFrame, "BOTTOMLEFT", 24, -3);
        cuiPetArtFrame.leftborder:SetTexture(texture);
        cuiPetArtFrame.leftborder:SetTexCoord(0.2402, 0.3027, 0.7910, 0.8750);
    end
    
    if (not cuiPetArtFrame.rightborder) then
        cuiPetArtFrame.rightborder = cuiPetArtFrame:CreateTexture("HIGHLIGHT");
        cuiPetArtFrame.rightborder:SetPoint("TOPLEFT", cuiPetArtFrame, "TOPRIGHT", 2, 3);
        cuiPetArtFrame.rightborder:SetPoint("BOTTOMRIGHT", cuiPetArtFrame, "BOTTOMRIGHT", -24, -3);
        cuiPetArtFrame.rightborder:SetTexture(texture);
        cuiPetArtFrame.rightborder:SetTexCoord(0.3027, 0.2402, 0.7910, 0.8750);
    end
end

function CleanUI_ReorderActionButtons()
    --ActionBarUpButton:Hide();
    --ActionBarDownButton:Hide();

    ActionBarUpButton:ClearAllPoints();
    ActionBarUpButton:SetPoint("TOPRIGHT", ActionButton1, "TOPLEFT", -12, 7);

    ActionBarDownButton:ClearAllPoints();
    ActionBarDownButton:SetPoint("BOTTOMRIGHT", ActionButton1, "BOTTOMLEFT", -12, -9);
    
    MainMenuBar:SetWidth(772);

    MultiBarBottomRightButton7:ClearAllPoints();
    MultiBarBottomRightButton7:SetPoint("TOPLEFT", ActionButton12, "TOPRIGHT", 12, 0);
    
    MainMenuBarBackpackButton:ClearAllPoints();
    MainMenuBarBackpackButton:SetPoint("BOTTOMRIGHT", UIParent, "BOTTOMRIGHT", -1, 1);
    
    MainMenuBarBackpackButton:SetScale(0.8)
    CharacterBag0Slot:SetScale(0.8)
    CharacterBag1Slot:SetScale(0.8)
    CharacterBag2Slot:SetScale(0.8)
    CharacterBag3Slot:SetScale(0.8)
end

function CleanUI_MainMenuTrackingBar_Configure(frame, isOnTop)
    CleanUI_ReorderExpBar();
end

function CleanUI_ReorderExpBar()
    MainMenuExpBar:SetSize(765, 6);
    MainMenuExpBar:SetPoint("TOP", 0, 0);
    MainMenuExpBar.forceHideText = true;

    ExhaustionLevelFillBar:ClearAllPoints();
    ExhaustionLevelFillBar:SetPoint("TOPLEFT", MainMenuExpBar, "TOPLEFT", 0, 0);
    ExhaustionLevelFillBar:SetPoint("BOTTOMRIGHT", MainMenuExpBar, "BOTTOMRIGHT", 0, 0);

    --ExhaustionTick:Hide();

    ReputationWatchBar:SetWidth(765);
    ReputationWatchBar.StatusBar.XPBarTexture0:Hide();
    ReputationWatchBar.StatusBar.XPBarTexture1:Hide();
    ReputationWatchBar.StatusBar.XPBarTexture2:Hide();
    ReputationWatchBar.StatusBar.XPBarTexture3:Hide(); 
    ReputationWatchBar.StatusBar.WatchBarTexture0:Hide(); 
    ReputationWatchBar.StatusBar.WatchBarTexture1:Hide(); 
    ReputationWatchBar.StatusBar.WatchBarTexture2:Hide(); 
    ReputationWatchBar.StatusBar.WatchBarTexture3:Hide();
    
    ReputationWatchBar.StatusBar:SetSize(765, 6);
    ReputationWatchBar.StatusBar:SetPoint("TOP", 0, 0);
    
    ArtifactWatchBar:SetWidth(765);
    ArtifactWatchBar.StatusBar.XPBarTexture0:Hide();
    ArtifactWatchBar.StatusBar.XPBarTexture1:Hide();
    ArtifactWatchBar.StatusBar.XPBarTexture2:Hide();
    ArtifactWatchBar.StatusBar.XPBarTexture3:Hide(); 
    ArtifactWatchBar.StatusBar.WatchBarTexture0:Hide(); 
    ArtifactWatchBar.StatusBar.WatchBarTexture1:Hide(); 
    ArtifactWatchBar.StatusBar.WatchBarTexture2:Hide(); 
    ArtifactWatchBar.StatusBar.WatchBarTexture3:Hide();
    
    ArtifactWatchBar.StatusBar:SetSize(765, 6);
    ArtifactWatchBar.StatusBar:SetPoint("TOP", 0, -1);
end

function CleanUI_ActionButton_OnUpdate(self, elapsed)
    local valid = IsActionInRange(self.action);
    
    if (valid ~= false) then
        self.icon:SetDesaturated(nil);
        self.icon:SetVertexColor(1.0, 1.0, 1.0);
    else
        self.icon:SetDesaturated(1);
        self.icon:SetVertexColor(1.0, 0.0, 0.0);
    end
    
    if (valid ~= false) then
        local isUsable, notEnoughMana = IsUsableAction(self.action);
        
        if (notEnoughMana) then
            local powerType, powerTypeString = UnitPowerType("player");
            local color = PowerBarColor[powerTypeString];
            
            self.icon:SetDesaturated(1);
            --self.icon:SetVertexColor(0.1, 0.25, 1.0);
            self.icon:SetVertexColor(color.r/2, color.g/2, color.b/2);
        elseif (not isUsable) then
            self.icon:SetDesaturated(1);
            self.icon:SetVertexColor(0.0, 0.4, 0.2);
        else
            self.icon:SetDesaturated(nil);
            self.icon:SetVertexColor(1.0, 1.0, 1.0);
        end
    end
end