local gcd_starttime, gcd_duration;

function CleanUI_InitCastingBars()
    CleanUISavedVariables.CASTINGBARS_SHOWPETBAR = true;
    CleanUISavedVariables.CASTINGBARS_ALPHA = 1;
    CleanUISavedVariables.CASTINGBARS_WIDTH = 300;
    CleanUISavedVariables.CASTINGBARS_PLAYERHEIGHT = 24;
    CleanUISavedVariables.CASTINGBARS_IMAGESIZE = 36;
    CleanUISavedVariables.CASTINGBARS_PETHEIGHT = 9;
    CleanUISavedVariables.CASTINGBARS_GLOBALCOOLDOWN = true;
    CleanUISavedVariables.CASTINGBARS_GLOBALCOOLDOWN_HEIGHT = 2;
    CleanUISavedVariables.CASTINGBARS_LATENCY = true;

    CleanUI_InitCastingBar(CastingBarFrame, CleanUISavedVariables.CASTINGBARS_WIDTH, CleanUISavedVariables.CASTINGBARS_PLAYERHEIGHT, true);
    CastingBarFrame:SetAlpha(CleanUISavedVariables.CASTINGBARS_ALPHA);

    -- hide the bizzard pet bar
    PetCastingBarFrame:ClearAllPoints(); 
    PetCastingBarFrame:SetPoint("TOP", UIParent, "BOTTOM", 0, 10000);

    if (CleanUISavedVariables.CASTINGBARS_SHOWPETBAR) then
        CastingBarFrame_OnLoad(CleanUIPetCastingBarFrame, "pet", true, false);
        CleanUI_InitCastingBar(CleanUIPetCastingBarFrame, CleanUISavedVariables.CASTINGBARS_WIDTH, CleanUISavedVariables.CASTINGBARS_PETHEIGHT, false);
        CleanUIPetCastingBarFrame:SetAlpha(CleanUISavedVariables.CASTINGBARS_ALPHA);
    end

    if (CleanUISavedVariables.CASTINGBARS_GLOBALCOOLDOWN) then
        CleanUICastingBarGlobalCooldownFrame:SetWidth(CleanUISavedVariables.CASTINGBARS_WIDTH);
        CleanUICastingBarGlobalCooldownFrame:SetHeight(CleanUISavedVariables.CASTINGBARS_GLOBALCOOLDOWN_HEIGHT);
        CleanUICastingBarGlobalCooldownFrame:SetAlpha(CleanUISavedVariables.CASTINGBARS_ALPHA);
    end

    if (CleanUISavedVariables.CASTINGBARS_LATENCY) then
        CleanUICastingBarLatency:SetHeight(CleanUISavedVariables.CASTINGBARS_PLAYERHEIGHT);
        CleanUICastingBarLatency:SetAlpha(CleanUISavedVariables.CASTINGBARS_ALPHA);
    end

    CleanUI_CastingBarHookEvents();
    CleanUI_RegisterCastingBarEvents();
    
    
    CleanUI_StartDelay(3, CleanUI_HideCastingBars);
end

function CleanUI_RegisterCastingBarEvents()
    local frame = CreateFrame("Frame");

    frame:RegisterEvent("UNIT_SPELLCAST_START");
    frame:RegisterEvent("UNIT_SPELLCAST_SUCCEEDED");

    frame:HookScript("OnEvent", CleanUI_CastingBarOnEvent);
end

function CleanUI_CastingBarOnEvent(self, event, ...)
    if (event == "UNIT_SPELLCAST_START" or event == "UNIT_SPELLCAST_SUCCEEDED") then
        local unit, spell = ...;
        CleanUI_CastingBar_UpdateGlobalCooldown(unit, spell);
    end
end

function CleanUI_CastingBarHookEvents()
    if (not CleanUISavedVariables.CHANGE_CASTINGBARS) then
        return;
    end

    hooksecurefunc("PetCastingBarFrame_OnEvent", CleanUI_PetCastingBarFrame_OnEvent);

    hooksecurefunc("CastingBarFrame_OnEvent", CleanUI_CastingBarFrame_OnEvent);
    hooksecurefunc("CastingBarFrame_OnUpdate", CleanUI_CastingBarFrame_OnUpdate);
    hooksecurefunc("MirrorTimer_Show", CleanUI_MirrorTimer_Show);
end

function CleanUI_PetCastingBarFrame_OnEvent(self, event, ...)
    if (not CleanUISavedVariables.CHANGE_CASTINGBARS) then
        return;
    end

    local arg1 = ...;
    if ( event == "UNIT_PET" ) then
        if ( arg1 == "player" ) then
            self:Hide();
        end
    end
end

function CleanUI_CastingBarOnUpdate(self, elapsed)
    CleanUI_CastingBar_GlobalCooldown_OnUpdate();
    CleanUI_CastingBar_Latency_OnUpdate();
end

function CleanUI_CastingBar_UpdateGlobalCooldown(unit, spell)
    if (unit == "player") then
        local start, dur = GetSpellCooldown(spell);

        if (start and dur and dur > 0 and dur <= 1.5) then
            gcd_starttime = start;
            gcd_duration = dur;
        end
    end
end

function CleanUI_CastingBar_GlobalCooldown_OnUpdate()

    if (not InCombatLockdown()) then
        CleanUICastingBarGlobalCooldownFrame:Hide();
        return;
    end

    if (not CleanUISavedVariables.CASTINGBARS_GLOBALCOOLDOWN) then
        CleanUICastingBarGlobalCooldownFrame:Hide();
        return;
    end

    local percent = 0;
    if (gcd_starttime and gcd_duration) then
        percent = (GetTime() - gcd_starttime) / gcd_duration;
    end

    if (percent <= 0 or percent > 1) then
        percent = 0;
    end

    CleanUICastingBarGlobalCooldownFrame:Show();
    CleanUICastingBarGlobalCooldownFrame:SetValue(percent);

    local r = 0.333 + percent^4 * 2 / 3;
    local g = r;
    local b = r;

    CleanUICastingBarGlobalCooldownFrame:SetStatusBarColor(r, g, b);
end

function CleanUI_CastingBar_Latency_OnUpdate()
    CleanUICastingBarLatency:Show();

    local statusMin, statusMax = CastingBarFrame:GetMinMaxValues();
    local down, up, lagHome, lagWorld = GetNetStats();

    lagHome = lagHome/1000;
    lagWorld = lagWorld/1000;

    --local lag = lagHome + lagWorld;
    local lag = lagWorld;

    if (lag > statusMax) then
        lag = statusMax;
    end

    -- ok, tricky we need right to left...

    CleanUICastingBarLatency:SetMinMaxValues(0, 1);
    CleanUICastingBarLatency:SetValue(1);

    local width = 0;
    if (statusMax ~= 0) then
        width = CleanUISavedVariables.CASTINGBARS_WIDTH / statusMax * lag;
    end

    CleanUICastingBarLatency:SetWidth(width);
end

function CleanUI_UpdateCastingBars()
    CastingBarFrame:SetWidth(CleanUISavedVariables.CASTINGBARS_WIDTH);
    CastingBarFrame:SetHeight(CleanUISavedVariables.CASTINGBARS_PLAYERHEIGHT);
    CastingBarFrame:SetAlpha(CleanUISavedVariables.CASTINGBARS_ALPHA);
    CastingBarFrame:Show();

    CastingBarFrameIcon:SetWidth(CleanUISavedVariables.CASTINGBARS_IMAGESIZE);
    CastingBarFrameIcon:SetHeight(CleanUISavedVariables.CASTINGBARS_IMAGESIZE);
    CastingBarFrameIcon:SetAlpha(CleanUISavedVariables.CASTINGBARS_ALPHA);
    CastingBarFrameIcon:Show();

    CleanUIPetCastingBarFrame:SetWidth(CleanUISavedVariables.CASTINGBARS_WIDTH);
    CleanUIPetCastingBarFrame:SetHeight(CleanUISavedVariables.CASTINGBARS_PETHEIGHT);
    CleanUIPetCastingBarFrame:SetAlpha(CleanUISavedVariables.CASTINGBARS_ALPHA);
    CleanUIPetCastingBarFrame:Show();

    CleanUICastingBarGlobalCooldownFrame:SetWidth(CleanUISavedVariables.CASTINGBARS_WIDTH);
    CleanUICastingBarGlobalCooldownFrame:SetHeight(CleanUISavedVariables.CASTINGBARS_GLOBALCOOLDOWN_HEIGHT);
    CleanUICastingBarGlobalCooldownFrame:SetAlpha(CleanUISavedVariables.CASTINGBARS_ALPHA);
    CleanUICastingBarGlobalCooldownFrame:SetValue(1);
    CleanUICastingBarGlobalCooldownFrame:Show();

    CleanUICastingBarLatency:SetHeight(CleanUISavedVariables.CASTINGBARS_PLAYERHEIGHT);
    CleanUICastingBarLatency:SetAlpha(CleanUISavedVariables.CASTINGBARS_ALPHA);
end

function CleanUI_HideCastingBars()
    CastingBarFrame:Hide();
    CleanUIPetCastingBarFrame:Hide();
    CleanUICastingBarGlobalCooldownFrame:Hide();
end

function CleanUI_InitCastingBar(self, width, height, showicon)

    -- new skin
    --self:SetStatusBarTexture(CleanUISavedVariables.CUI_STATUSBAR);

    local name = self:GetName();

    local spellbarBorder = _G["CleanUI"..name.."BorderFrame"];
    if (not spellbarBorder) then
        spellbarBorder = CreateFrame("Frame", "CleanUI"..name.."BorderFrame", self);
    end

    spellbarBorder:SetPoint("TOPLEFT", -3, 3);
    spellbarBorder:SetPoint("BOTTOMRIGHT", 3, -2.5);
    spellbarBorder:SetFrameStrata("LOW");
    
    local spacex = 6;
    
    if (height < 20) then spacex = 3; end
    
    spellbarBorder.border = spellbarBorder:CreateTexture("BORDER");
    spellbarBorder.border:SetPoint("TOPLEFT", spellbarBorder, "TOPLEFT", -5, spacex);
    spellbarBorder.border:SetPoint("BOTTOMRIGHT", spellbarBorder, "BOTTOMRIGHT", 5, -spacex);
    spellbarBorder.border:SetTexture("Interface\\AddOns\\CleanUI\\skins\\castingbar\\border");

    if (showicon) then
        self.Icon:Show();
        self.Icon:SetWidth(CleanUISavedVariables.CASTINGBARS_IMAGESIZE);
        self.Icon:SetHeight(CleanUISavedVariables.CASTINGBARS_IMAGESIZE);

        self.Icon:SetTexCoord(0.1, 0.9, 0.1, 0.9);

        local spellbarIconBorder = _G["CleanUI"..name.."IconBorder"];
        if (not spellbarIconBorder) then
            spellbarIconBorder = CreateFrame("Frame", "CleanUI"..name.."IconBorder", self);
        end

        spellbarIconBorder:SetPoint("TOPLEFT", self.Icon, "TOPLEFT", -3, 3);
        spellbarIconBorder:SetPoint("BOTTOMRIGHT", self.Icon, "BOTTOMRIGHT", 3, -2);
        spellbarIconBorder:SetFrameStrata("HIGH");

        CleanUI_SetBackdrop(spellbarIconBorder, false);
    else
        self.Icon:Hide();
    end

    self.Border:SetTexture();

    self:SetSize(width, height);

    self.BorderShield:SetTexture();

    self.Text:ClearAllPoints();
    self.Text:SetPoint("CENTER", spellbarBorder, "CENTER", 0, 0);

    self.Spark:SetTexture("Interface\\AddOns\\CleanUI\\skins\\castingbar\\spark");
    self.Flash:SetTexture("Interface\\AddOns\\CleanUI\\skins\\castingbar\\flash");

    self.Spark:SetSize(30, height + 8);

    self.Flash:ClearAllPoints();
    self.Flash:SetPoint("TOPLEFT", spellbarBorder, "TOPLEFT", 2, -2);
    self.Flash:SetPoint("BOTTOMRIGHT", spellbarBorder, "BOTTOMRIGHT", -2, 2);
    
    -- timer
    local castingBarTimer = self:CreateFontString("CUI_"..name.."Timer", "ARTWORK");
    castingBarTimer:SetFontObject("GameFontNormalSmall");
    castingBarTimer:SetWidth(30);
    castingBarTimer:SetHeight(10);
    castingBarTimer:ClearAllPoints();
    castingBarTimer:SetPoint("RIGHT", spellbarBorder, "RIGHT", -6, 0);
    castingBarTimer:SetJustifyH("LEFT");
    castingBarTimer:SetText("");
end

function CleanUI_MirrorTimer_Show(timer, value, maxvalue, scale, paused, label)
    for index = 1, MIRRORTIMER_NUMTIMERS, 1 do
        local frame = _G["MirrorTimer"..index];
        CleanUI_InitMirrorTimer(frame);
    end
end

function CleanUI_InitMirrorTimer(self)

    local name = self:GetName();
    local text = _G[name.."Text"];
    local statusbar = _G[name.."StatusBar"];

    _G[name.."Border"]:SetTexture();

    local border = _G["CleanUI"..name.."Border"];
    if (not border) then
        border = CreateFrame("Frame", "CleanUI"..name.."Border", self);
    end

    border:SetPoint("TOPLEFT", -2, 2);
    border:SetPoint("BOTTOMRIGHT", 2, 8);
    border:SetFrameStrata("BACKGROUND");

    CleanUI_SetBackdrop(border, false);

    self:SetWidth(200);

    text:ClearAllPoints();
    text:SetPoint("CENTER", border, "CENTER", 0, 0);
end

function CleanUI_CastingBarFrame_OnEvent(self, event, ...)

    local unit = self.unit;

    if (unit and UnitExists(unit)) then
        local border = nil;

        if (unit == "player") then
            border = _G["CleanUIPlayerSpellbarBorder"];
        elseif (unit == "target") then
            border = _G["CleanUITargetSpellbarBorder"];
        elseif (unit == "focus") then
            border = _G["CleanUIFocusSpellbarBorder"];
        end

        if (border) then
            local name, nameSubtext, text, texture, startTime, endTime, isTradeSkill, castID, notInterruptible;

            if (event == "UNIT_SPELLCAST_START") then
                name, nameSubtext, text, texture, startTime, endTime, isTradeSkill, castID, notInterruptible = UnitCastingInfo(unit);
            end

            if (event == "UNIT_SPELLCAST_CHANNEL_START"
                or event == "UNIT_SPELLCAST_CHANNEL_UPDATE"
                or event == "UNIT_SPELLCAST_DELAYED"
                or event == "UNIT_SPELLCAST_INTERRUPTIBLE"
                or event == "UNIT_SPELLCAST_NOT_INTERRUPTIBLE") then
                name, nameSubtext, text, texture, startTime, endTime, isTradeSkill, notInterruptible = UnitChannelInfo(unit);
            end

            if ( notInterruptible ) then
                border:SetBackdropBorderColor(1.0, 0.0, 0.0, 1.0);
            else
                border:SetBackdropBorderColor(1.0, 1.0, 1.0, 1.0);
            end
        end
    end

    if ( event == "UNIT_SPELLCAST_START" ) then
        self:SetAlpha(CleanUISavedVariables.CASTINGBARS_ALPHA);
    elseif ( event == "UNIT_SPELLCAST_CHANNEL_START" ) then
        self:SetAlpha(CleanUISavedVariables.CASTINGBARS_ALPHA);
    end
end

function CleanUI_CastingBarFrame_OnUpdate(self, elapsed)

    local name = self:GetName();

    local castingBarTimer = _G["CUI_"..name.."Timer"];

    if (not castingBarTimer) then
        return;
    end

    local value = 0;

    if (self.casting) then
        value = self.maxValue - self.value;
    elseif (self.channeling) then
        value = self.value;
    end

    if (value >= 0) then
        castingBarTimer:SetText(format("%.1f", value));
    else
        castingBarTimer:SetText("");
    end
end
