
local texture = "Interface\\PLAYERACTIONBARALT\\GENERICWOW";

function CleanUI_InitChat()

    CleanUI_ChatHookEvents();
    CleanUI_RegisterChatEvents();
    
    CleanUI_DisableAltArrowKeyMode();
    
    --ChatFrame1:SetClampRectInsets(-35, 35, 26, -29);

    QuickJoinToastButton:ClearAllPoints();
    QuickJoinToastButton:SetPoint("TOP", ChatFrame1ButtonFrameBottomButton, "BOTTOM", 0, -9);
    
    CleanUI_InitChatFrameArtwork();
end

function CleanUI_ChatFrameAddMessage(frame, msg, ...)
    msg = "-"..msg;
    frame.__orig__AddMessage(frame, msg, ...);
end

function CleanUI_RegisterChatEvents()
    local frame = CreateFrame("Frame");

    frame:RegisterEvent("ADDON_LOADED");
    --frame:RegisterEvent("MERCHANT_SHOW");
    --frame:RegisterEvent("BANKFRAME_OPENED");
    --frame:RegisterEvent("BAG_UPDATE");

    frame:HookScript("OnEvent", CleanUI_ChatOnEvent);
end

function CleanUI_ChatOnEvent(self, event, ...)

    if (event == "ADDON_LOADED") then
        local addonName = ...;
        if ( addonName == "..." ) then
        end
    end

    --if (event == "MERCHANT_SHOW") then
        --CleanUI_MerchantShow();
    --end
end

function CleanUI_ChatHookEvents()
    hooksecurefunc("ChatEdit_ActivateChat", CleanUI_ChatEdit_ActivateChat);
    hooksecurefunc("ChatEdit_DeactivateChat", CleanUI_ChatEdit_DeactivateChat);
    hooksecurefunc("FCF_GetChatWindowInfo", CleanUI_FCF_GetChatWindowInfo);
    
    hooksecurefunc("ChatEdit_OnChar", CleanUI_ChatEdit_OnChar);
end

function CleanUI_ChatEdit_OnChar(self)
    local text = self:GetText();
    
    local newText = gsub(text, "%:%-%)", "☺");
    newText = gsub(newText, "%;%-%)", "☻");
    newText = gsub(newText, "%<3", "♥");
    
    self:SetText(newText);
end

function CleanUI_DisableAltArrowKeyMode()
    local i = 1;
    local editbox = "ChatFrame"..i.."EditBox";

    while ( _G[editbox] ~= nil ) do
        _G[editbox]:SetAltArrowKeyMode(nil);
        i = i + 1;
        editbox = "ChatFrame"..i.."EditBox";
    end
end

function CleanUI_ChatEdit_ActivateChat(editBox)
    editBox:Show();
    editBox:SetFrameStrata("DIALOG");
    editBox:Raise();
    
    editBox.header:Show();
    editBox.focusLeft:Show();
    editBox.focusRight:Show();
    editBox.focusMid:Show();
    editBox:SetAlpha(1.0);
end

function CleanUI_ChatEdit_DeactivateChat(editBox)
    editBox:Show();
    editBox:SetFrameStrata("DIALOG");
    editBox:Raise();
    
    editBox.header:Show();
    editBox.focusLeft:Show();
    editBox.focusRight:Show();
    editBox.focusMid:Show();
    editBox:SetAlpha(0.3);
end

function CleanUI_InitChatFrameArtwork()
    local editbox;
    
    for i = 1, 20 do   
        CleanUI_InitChatFrameEditBoxArtwork(i);
    
        editbox = _G["ChatFrame"..i.."EditBox"];
        if (editbox) then
            hooksecurefunc(editbox, "Hide", CleanUI_ChatFrameEditBoxHide);
        end
    end
end

function CleanUI_ChatFrameEditBoxHide(editbox) 
    CleanUI_ChatEdit_DeactivateChat(editbox);
end

function CleanUI_FCF_GetChatWindowInfo(id)
    if ( id > NUM_CHAT_WINDOWS ) then
        CleanUI_InitChatFrameEditBoxArtwork(id);
    end
end

function CleanUI_InitChatFrameEditBoxArtwork(index) 
    local chatframe = _G["ChatFrame"..index];
    local editbox = _G["ChatFrame"..index.."EditBox"];
    
    if (chatframe == nil) then
        return
    end
    
    chatframe:SetFrameStrata("HIGH");
    editbox:SetFrameStrata("HIGH");
    
    CleanUI_ChatEdit_DeactivateChat(editbox);
    
    _G["ChatFrame"..index.."EditBoxLeft"]:Hide();
    _G["ChatFrame"..index.."EditBoxRight"]:Hide();
    _G["ChatFrame"..index.."EditBoxMid"]:Hide();
    
    -- graphics
    local artFrame = _G["ChatFrame"..index.."EditBoxArtFrame"];
    if (not artFrame) then
        artFrame = CreateFrame("Frame", "ChatFrame"..index.."EditBoxArtFrame", ChatFrame1);
    end

    artFrame:SetParent(chatframe);
    artFrame:SetPoint("TOPLEFT", chatframe, "BOTTOMLEFT", -3, -8);
    artFrame:SetPoint("TOPRIGHT", chatframe, "BOTTOMRIGHT", 3, -8);
    artFrame:SetHeight(50);
    artFrame:SetFrameStrata("BACKGROUND");
    
    editbox:SetParent(artFrame);
    editbox:ClearAllPoints();
    editbox:SetHeight(50);
    editbox:SetPoint("TOPLEFT", chatframe, "BOTTOMLEFT", 3, -8);
    editbox:SetPoint("TOPRIGHT", chatframe, "BOTTOMRIGHT", -4, -8);
    
    if (not artFrame.background) then
        artFrame.background = artFrame:CreateTexture("BACKGROUND");
        artFrame.background:SetPoint("TOPLEFT", artFrame, "TOPLEFT", 0, -5);
        artFrame.background:SetPoint("BOTTOMRIGHT", artFrame, "BOTTOMRIGHT", 0, 7);
        artFrame.background:SetTexture(texture);
        artFrame.background:SetTexCoord(0.0, 1.0, 0.1757, 0.3555);
    end    
    
    if (not artFrame.topborder) then
        artFrame.topborder = artFrame:CreateTexture("HIGHLIGHT");
        artFrame.topborder:SetPoint("TOPLEFT", artFrame, "TOPLEFT", 0, -5);
        artFrame.topborder:SetPoint("BOTTOMRIGHT", artFrame, "TOPRIGHT", 0, -8);
        artFrame.topborder:SetTexture(texture);
        artFrame.topborder:SetTexCoord(0.0, 1.0, 0.0898, 0.0977);
    end
    
    if (not artFrame.leftborder) then
        artFrame.leftborder = artFrame:CreateTexture("HIGHLIGHT");
        artFrame.leftborder:SetPoint("TOPLEFT", artFrame, "TOPLEFT", 0, -2);
        artFrame.leftborder:SetPoint("BOTTOMRIGHT", artFrame, "BOTTOMLEFT", 25, 7);
        artFrame.leftborder:SetTexture(texture);
        artFrame.leftborder:SetTexCoord(0.2402, 0.3027, 0.7910, 0.8645);
    end
    
    if (not artFrame.rightborder) then
        artFrame.rightborder = artFrame:CreateTexture("HIGHLIGHT");
        artFrame.rightborder:SetPoint("TOPLEFT", artFrame, "TOPRIGHT", 0, -2);
        artFrame.rightborder:SetPoint("BOTTOMRIGHT", artFrame, "BOTTOMRIGHT", -25, 7);
        artFrame.rightborder:SetTexture(texture);
        artFrame.rightborder:SetTexCoord(0.3027, 0.2402, 0.7910, 0.8645);
    end
end